@@ -3,19 +3,24 @@
 // All rights reserved. Distributed under LGPL 3.0. For full terms see the file LICENSE.
 
 use axes_common::*;
-use datatype::*;
 use coordinates::*;
+use datatype::*;
 use options::*;
 use writer::Writer;
 
 /// 2D axes that is used for drawing 2D plots
-pub struct Axes2D
+pub struct Axes2D<'l>
 {
-	common: AxesCommonData,
+	common: AxesCommonData<'l>,
 }
 
-impl Axes2D
+impl<'m> Axes2D<'m>
 {
+	pub(crate) fn new() -> Axes2D<'m>
+	{
+		Axes2D { common: AxesCommonData::new() }
+	}
+
 	/// Sets the properties of the plot border
 	///
 	/// # Arguments
@@ -26,7 +31,7 @@ impl Axes2D
 	///      * `Color` - Specifies the color of the border
 	///      * `LineStyle` - Specifies the style of the border
 	///      * `LineWidth` - Specifies the width of the border
-	pub fn set_border<'l>(&'l mut self, front: bool, locations: &[BorderLocation2D], options: &[PlotOption]) -> &'l mut Axes2D
+	pub fn set_border<'l>(&'l mut self, front: bool, locations: &[BorderLocation2D], options: &[PlotOption<'m>]) -> &'l mut Self
 	{
 		{
 			let c = &mut self.common.commands as &mut Writer;
@@ -37,7 +42,7 @@ impl Axes2D
 				f |= l as i32;
 			}
 			write!(c, "{}", f);
-			c.write_str( if front
+			c.write_str(if front
 			{
 				" front "
 			}
@@ -54,7 +59,7 @@ impl Axes2D
 		self
 	}
 
-	fn set_axis_common<'l>(&'l mut self, axis: &str, show: bool, options: &[PlotOption]) -> &'l mut Axes2D
+	fn set_axis_common<'l>(&'l mut self, axis: &str, show: bool, options: &[PlotOption<'m>]) -> &'l mut Self
 	{
 		{
 			let c = &mut self.common.commands as &mut Writer;
@@ -83,17 +88,17 @@ impl Axes2D
 	/// # Arguments
 	///
 	/// * `show` - Whether or not draw the axis
-	/// * `options` - Array of PlotOption controlling the appearance of the axis. Relevant options are:
+	/// * `options` - Array of PlotOption<'m> controlling the appearance of the axis. Relevant options are:
 	///      * `Color` - Specifies the color of the border
 	///      * `LineStyle` - Specifies the style of the border
 	///      * `LineWidth` - Specifies the width of the border
-	pub fn set_x_axis<'l>(&'l mut self, show: bool, options: &[PlotOption]) -> &'l mut Axes2D
+	pub fn set_x_axis<'l>(&'l mut self, show: bool, options: &[PlotOption<'m>]) -> &'l mut Self
 	{
 		self.set_axis_common("x", show, options)
 	}
 
 	/// Like `set_x_axis` but for the y axis.
-	pub fn set_y_axis<'l>(&'l mut self, show: bool, options: &[PlotOption]) -> &'l mut Axes2D
+	pub fn set_y_axis<'l>(&'l mut self, show: bool, options: &[PlotOption<'m>]) -> &'l mut Self
 	{
 		self.set_axis_common("y", show, options)
 	}
@@ -104,13 +109,14 @@ impl Axes2D
 	/// * `y1` - Y coordinate of the arrow start
 	/// * `x2` - X coordinate of the arrow end
 	/// * `y2` - Y coordinate of the arrow end
-	/// * `options` - Array of PlotOption controlling the appearance of the arrowhead and arrow shaft. Relevant options are:
+	/// * `options` - Array of PlotOption<'m> controlling the appearance of the arrowhead and arrow shaft. Relevant options are:
 	///      * `ArrowType` - Specifies the style of the arrow head (or an option to omit it)
 	///      * `ArrowSize` - Sets the size of the arrow head (in graph units)
 	///      * `Color` - Specifies the color of the arrow
 	///      * `LineStyle` - Specifies the style of the arrow shaft
 	///      * `LineWidth` - Specifies the width of the arrow shaft
-	pub fn arrow<'l>(&'l mut self, x1: Coordinate, y1: Coordinate, x2: Coordinate, y2: Coordinate, options: &[PlotOption]) -> &'l mut Axes2D
+	pub fn arrow<'l>(&'l mut self, x1: Coordinate, y1: Coordinate, x2: Coordinate, y2: Coordinate, options: &[PlotOption<'m>])
+		-> &'l mut Self
 	{
 		{
 			let c = &mut self.common.commands as &mut Writer;
@@ -160,7 +166,8 @@ impl Axes2D
 	///     * `TextColor`
 	///     * `TextAlign(AlignLeft)`
 	///     * `TextAlign(AlignRight)`
-	pub fn set_legend<'l>(&'l mut self, x: Coordinate, y: Coordinate, legend_options: &'l [LegendOption], text_options: &'l [LabelOption]) -> &'l mut Axes2D
+	pub fn set_legend<'l>(&'l mut self, x: Coordinate, y: Coordinate, legend_options: &[LegendOption], text_options: &[LabelOption])
+		-> &'l mut Self
 	{
 		{
 			let c = &mut self.common.commands as &mut Writer;
@@ -181,7 +188,8 @@ impl Axes2D
 						AlignTop => " top",
 						AlignBottom => " bottom",
 						_ => " center"
-					});				},
+					});
+				},
 				_ =>
 				{
 					c.write_str(" right top");
@@ -283,14 +291,15 @@ impl Axes2D
 	/// # Arguments
 	/// * `x` - x values
 	/// * `y` - y values
-	/// * `options` - Array of PlotOption controlling the appearance of the plot element. The relevant options are:
+	/// * `options` - Array of PlotOption<'m> controlling the appearance of the plot element. The relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
 	///     * `LineWidth` - Sets the width of the line
 	///     * `LineStyle` - Sets the style of the line
 	///     * `Color` - Sets the color
-	pub fn lines<'l, Tx: DataType, X: IntoIterator<Item = Tx>, Ty: DataType, Y: IntoIterator<Item = Ty>>(&'l mut self, x: X, y: Y, options: &[PlotOption]) -> &'l mut Axes2D
+	pub fn lines<'l, Tx: DataType, X: IntoIterator<Item = Tx>, Ty: DataType, Y: IntoIterator<Item = Ty>>(&'l mut self, x: X, y: Y, options: &[PlotOption<'m>])
+		-> &'l mut Self
 	{
-		self.common.plot2(Lines, x, y, options);
+		self.common.elems.push(PlotElement::new_plot2(Lines, x, y, options.to_vec()));
 		self
 	}
 
@@ -298,14 +307,15 @@ impl Axes2D
 	/// # Arguments
 	/// * `x` - x values
 	/// * `y` - y values
-	/// * `options` - Array of PlotOption controlling the appearance of the plot element. The relevant options are:
+	/// * `options` - Array of PlotOption<'m> controlling the appearance of the plot element. The relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
 	///     * `PointSymbol` - Sets symbol for each point
 	///     * `PointSize` - Sets the size of each point
 	///     * `Color` - Sets the color
-	pub fn points<'l, Tx: DataType, X: IntoIterator<Item = Tx>, Ty: DataType, Y: IntoIterator<Item = Ty>>(&'l mut self, x: X, y: Y, options: &[PlotOption]) -> &'l mut Axes2D
+	pub fn points<'l, Tx: DataType, X: IntoIterator<Item = Tx>, Ty: DataType, Y: IntoIterator<Item = Ty>>(&'l mut self, x: X, y: Y, options: &[PlotOption<'m>])
+		-> &'l mut Self
 	{
-		self.common.plot2(Points, x, y, options);
+		self.common.elems.push(PlotElement::new_plot2(Points, x, y, options.to_vec()));
 		self
 	}
 
@@ -313,10 +323,65 @@ impl Axes2D
 	/// # Arguments
 	/// * `x` - x values
 	/// * `y` - y values
-	/// * `options` - Array of PlotOption controlling the appearance of the plot element
-	pub fn lines_points<'l, Tx: DataType, X: IntoIterator<Item = Tx>, Ty: DataType, Y: IntoIterator<Item = Ty>>(&'l mut self, x: X, y: Y, options: &[PlotOption]) -> &'l mut Axes2D
+	/// * `options` - Array of PlotOption<'m> controlling the appearance of the plot element
+	pub fn lines_points<'l, Tx: DataType, X: IntoIterator<Item = Tx>, Ty: DataType, Y: IntoIterator<Item = Ty>>(&'l mut self, x: X, y: Y, options: &[PlotOption<'m>])
+		-> &'l mut Self
 	{
-		self.common.plot2(LinesPoints, x, y, options);
+		self.common.elems.push(PlotElement::new_plot2(LinesPoints, x, y, options.to_vec()));
+		self
+	}
+
+	/// Plot a 2D scatter-plot with a point standing in for each data point.
+	/// Additionally, error bars are attached to each data point in the X direction.
+	/// # Arguments
+	/// * `x` - x values
+	/// * `y` - y valuess
+	/// * `x_error` - Errors associated with the x value
+	/// * `options` - Array of PlotOption controlling the appearance of the plot element. The relevant options are:
+	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
+	///     * `PointSymbol` - Sets symbol for each point
+	///     * `PointSize` - Sets the size of each point
+	///     * `Color` - Sets the color
+	pub fn x_error_bars<
+		'l,
+		Tx: DataType,
+		X: IntoIterator<Item = Tx>,
+		Ty: DataType,
+		Y: IntoIterator<Item = Ty>,
+		Txe: DataType,
+		XE: IntoIterator<Item = Txe>,
+	>(
+		&'l mut self, x: X, y: Y, x_error: XE, options: &[PlotOption<'m>]
+	) -> &'l mut Self
+	{
+		self.common.elems.push(PlotElement::new_plot3(XErrorBars, x, y, x_error, options.to_vec()));
+		self
+	}
+
+	/// Plot a 2D scatter-plot with a point standing in for each data point.
+	/// Additionally, error bars are attached to each data point in the Y direction.
+	/// # Arguments
+	/// * `x` - x values
+	/// * `y` - y values
+	/// * `y_error` - Errors associated with the y values
+	/// * `options` - Array of PlotOption<'m> controlling the appearance of the plot element. The relevant options are:
+	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
+	///     * `PointSymbol` - Sets symbol for each point
+	///     * `PointSize` - Sets the size of each point
+	///     * `Color` - Sets the color
+	pub fn y_error_bars<
+		'l,
+		Tx: DataType,
+		X: IntoIterator<Item = Tx>,
+		Ty: DataType,
+		Y: IntoIterator<Item = Ty>,
+		Tye: DataType,
+		YE: IntoIterator<Item = Tye>,
+	>(
+		&'l mut self, x: X, y: Y, y_error: YE, options: &[PlotOption<'m>]
+	) -> &'l mut Self
+	{
+		self.common.elems.push(PlotElement::new_plot3(YErrorBars, x, y, y_error, options.to_vec()));
 		self
 	}
 
@@ -333,12 +398,19 @@ impl Axes2D
 	///     * `LineWidth` - Sets the width of the line
 	///     * `LineStyle` - Sets the style of the line
 	///     * `Color` - Sets the color
-	pub fn x_error_lines<'l,
-	                   Tx: DataType, X: IntoIterator<Item = Tx>,
-	                   Ty: DataType, Y: IntoIterator<Item = Ty>,
-	                   Txe: DataType, XE: IntoIterator<Item = Txe>>(&'l mut self, x: X, y: Y, x_error: XE, options: &[PlotOption]) -> &'l mut Axes2D
+	pub fn x_error_lines<
+		'l,
+		Tx: DataType,
+		X: IntoIterator<Item = Tx>,
+		Ty: DataType,
+		Y: IntoIterator<Item = Ty>,
+		Txe: DataType,
+		XE: IntoIterator<Item = Txe>,
+	>(
+		&'l mut self, x: X, y: Y, x_error: XE, options: &[PlotOption<'m>]
+	) -> &'l mut Self
 	{
-		self.common.plot3(XErrorLines, x, y, x_error, options);
+		self.common.elems.push(PlotElement::new_plot3(XErrorLines, x, y, x_error, options.to_vec()));
 		self
 	}
 
@@ -348,19 +420,26 @@ impl Axes2D
 	/// * `x` - x values
 	/// * `y` - y values
 	/// * `y_error` - Errors associated with the y values
-	/// * `options` - Array of PlotOption controlling the appearance of the plot element. The relevant options are:
+	/// * `options` - Array of PlotOption<'m> controlling the appearance of the plot element. The relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
 	///     * `PointSymbol` - Sets symbol for each point
 	///     * `PointSize` - Sets the size of each point
 	///     * `LineWidth` - Sets the width of the line
 	///     * `LineStyle` - Sets the style of the line
 	///     * `Color` - Sets the color
-	pub fn y_error_lines<'l,
-	                   Tx: DataType, X: IntoIterator<Item = Tx>,
-	                   Ty: DataType, Y: IntoIterator<Item = Ty>,
-	                   Tye: DataType, YE: IntoIterator<Item = Tye>>(&'l mut self, x: X, y: Y, y_error: YE, options: &[PlotOption]) -> &'l mut Axes2D
+	pub fn y_error_lines<
+		'l,
+		Tx: DataType,
+		X: IntoIterator<Item = Tx>,
+		Ty: DataType,
+		Y: IntoIterator<Item = Ty>,
+		Tye: DataType,
+		YE: IntoIterator<Item = Tye>,
+	>(
+		&'l mut self, x: X, y: Y, y_error: YE, options: &[PlotOption<'m>]
+	) -> &'l mut Self
 	{
-		self.common.plot3(YErrorLines, x, y, y_error, options);
+		self.common.elems.push(PlotElement::new_plot3(YErrorLines, x, y, y_error, options.to_vec()));
 		self
 	}
 
@@ -371,17 +450,24 @@ impl Axes2D
 	/// * `x` - x values
 	/// * `y_lo` - Bottom y values
 	/// * `y_hi` - Top y values
-	/// * `options` - Array of PlotOption controlling the appearance of the plot element. The relevant options are:
+	/// * `options` - Array of PlotOption<'m> controlling the appearance of the plot element. The relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
 	///     * `FillRegion` - Specifies the region between the two curves to fill
 	///     * `Color` - Sets the color of the filled region
 	///     * `FillAlpha` - Sets the transparency of the filled region
-	pub fn fill_between<'l,
-	                   Tx: DataType, X: IntoIterator<Item = Tx>,
-	                   Tyl: DataType, YL: IntoIterator<Item = Tyl>,
-	                   Tyh: DataType, YH: IntoIterator<Item = Tyh>>(&'l mut self, x: X, y_lo: YL, y_hi: YH, options: &[PlotOption]) -> &'l mut Axes2D
+	pub fn fill_between<
+		'l,
+		Tx: DataType,
+		X: IntoIterator<Item = Tx>,
+		Tyl: DataType,
+		YL: IntoIterator<Item = Tyl>,
+		Tyh: DataType,
+		YH: IntoIterator<Item = Tyh>,
+	>(
+		&'l mut self, x: X, y_lo: YL, y_hi: YH, options: &[PlotOption<'m>]
+	) -> &'l mut Self
 	{
-		self.common.plot3(FillBetween, x, y_lo, y_hi, options);
+		self.common.elems.push(PlotElement::new_plot3(FillBetween, x, y_lo, y_hi, options.to_vec()));
 		self
 	}
 
@@ -390,16 +476,17 @@ impl Axes2D
 	/// # Arguments
 	/// * `x` - x values (center of the box)
 	/// * `y` - y values
-	/// * `options` - Array of PlotOption controlling the appearance of the plot element. The relevant options are:
+	/// * `options` - Array of PlotOption<'m> controlling the appearance of the plot element. The relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
 	///     * `LineWidth` - Sets the width of the border
 	///     * `LineStyle` - Sets the style of the border
 	///     * `BorderColor` - Sets the color of the border
 	///     * `Color` - Sets the color of the box fill
 	///     * `FillAlpha` - Sets the transparency of the box fill
-	pub fn boxes<'l, Tx: DataType, X: IntoIterator<Item = Tx>, Ty: DataType, Y: IntoIterator<Item = Ty>>(&'l mut self, x: X, y: Y, options: &[PlotOption]) -> &'l mut Axes2D
+	pub fn boxes<'l, Tx: DataType, X: IntoIterator<Item = Tx>, Ty: DataType, Y: IntoIterator<Item = Ty>>(&'l mut self, x: X, y: Y, options: &[PlotOption<'m>])
+		-> &'l mut Self
 	{
-		self.common.plot2(Boxes, x, y, options);
+		self.common.elems.push(PlotElement::new_plot2(Boxes, x, y, options.to_vec()));
 		self
 	}
 
@@ -409,22 +496,26 @@ impl Axes2D
 	/// * `x` - x values (center of the box)
 	/// * `y` - y values
 	/// * `w` - Box width values
-	/// * `options` - Array of PlotOption controlling the appearance of the plot element. The relevant options are:
+	/// * `options` - Array of PlotOption<'m> controlling the appearance of the plot element. The relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
 	///     * `LineWidth` - Sets the width of the border
 	///     * `LineStyle` - Sets the style of the border
 	///     * `BorderColor` - Sets the color of the border
 	///     * `Color` - Sets the color of the box fill
 	///     * `FillAlpha` - Sets the transparency of the box fill
-	pub fn boxes_set_width<'l,
-	                      Tx: DataType,
-	                      X: IntoIterator<Item = Tx>,
-	                      Ty: DataType,
-	                      Y: IntoIterator<Item = Ty>,
-	                      Tw: DataType,
-	                      W: IntoIterator<Item = Tw>>(&'l mut self, x: X, y: Y, w: W, options: &[PlotOption]) -> &'l mut Axes2D
+	pub fn boxes_set_width<
+		'l,
+		Tx: DataType,
+		X: IntoIterator<Item = Tx>,
+		Ty: DataType,
+		Y: IntoIterator<Item = Ty>,
+		Tw: DataType,
+		W: IntoIterator<Item = Tw>,
+	>(
+		&'l mut self, x: X, y: Y, w: W, options: &[PlotOption<'m>]
+	) -> &'l mut Self
 	{
-		self.common.plot3(Boxes, x, y, w, options);
+		self.common.elems.push(PlotElement::new_plot3(Boxes, x, y, w, options.to_vec()));
 		self
 	}
 
@@ -437,48 +528,39 @@ impl Axes2D
 	/// * `num_cols` - Number of columns in the data array
 	/// * `dimensions` - Optional X and Y coordinates of the first and last data points (with the rest of the coordinates spaced evenly between).
 	///                  By default this will be `(0, 0)` and `(num_rows - 1, num_cols - 1)`.
-	/// * `options` - Array of PlotOption controlling the appearance of the surface. Relevant options are:
+	/// * `options` - Array of PlotOption<'m> controlling the appearance of the surface. Relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
-	pub fn image<'l,
-	              T: DataType,
-	              X: IntoIterator<Item = T>>(&'l mut self, mat: X, num_rows: usize, num_cols: usize, dimensions: Option<(f64, f64, f64, f64)>,
-	                              options: &[PlotOption]) -> &'l mut Axes2D
+	pub fn image<'l, T: DataType, X: IntoIterator<Item = T>>(&'l mut self, mat: X, num_rows: usize, num_cols: usize, dimensions: Option<(f64, f64, f64, f64)>, options: &[PlotOption<'m>])
+		-> &'l mut Self
 	{
-		self.common.plot_matrix(Image, false, mat, num_rows, num_cols, dimensions, options);
+		self.common
+			.elems
+			.push(PlotElement::new_plot_matrix(Image, false, mat, num_rows, num_cols, dimensions, options.to_vec()));
 		self
 	}
 }
 
-impl AxesCommonPrivate for Axes2D
+impl<'l> AxesCommonPrivate<'l> for Axes2D<'l>
 {
-	fn get_common_data_mut<'l>(&'l mut self) -> &'l mut AxesCommonData
+	fn get_common_data_mut(&mut self) -> &mut AxesCommonData<'l>
 	{
 		&mut self.common
 	}
 
-	fn get_common_data<'l>(&'l self) -> &'l AxesCommonData
+	fn get_common_data(&self) -> &AxesCommonData<'l>
 	{
 		&self.common
 	}
 }
 
-impl AxesCommon for Axes2D {}
+impl<'l> AxesCommon<'l> for Axes2D<'l> {}
 
-pub fn new_axes2d() -> Axes2D
-{
-	Axes2D
-	{
-		common: AxesCommonData::new(),
-	}
-}
-
-#[doc(hidden)]
-pub trait Axes2DPrivate
+pub(crate) trait Axes2DPrivate
 {
 	fn write_out(&self, writer: &mut Writer);
 }
 
-impl Axes2DPrivate for Axes2D
+impl<'l> Axes2DPrivate for Axes2D<'l>
 {
 	fn write_out(&self, writer: &mut Writer)
 	{
