@@ -2,18 +2,18 @@
 //
 // All rights reserved. Distributed under LGPL 3.0. For full terms see the file LICENSE.
 
-use std::io::Write;
 
 use axes_common::*;
 use datatype::*;
 use options::*;
+use std::io::Write;
 use writer::Writer;
 
 /// 3D axes that is used for drawing 3D plots
-pub struct Axes3D
+pub struct Axes3D<'l>
 {
-	common: AxesCommonData,
-	z_axis: AxisData,
+	common: AxesCommonData<'l>,
+	z_axis: AxisData<'l>,
 	contour_base: bool,
 	contour_surface: bool,
 	contour_auto: AutoOption<u32>,
@@ -22,8 +22,22 @@ pub struct Axes3D
 	contour_label: AutoOption<String>,
 }
 
-impl Axes3D
+impl<'m> Axes3D<'m>
 {
+	pub(crate) fn new() -> Axes3D<'m>
+	{
+		Axes3D {
+			common: AxesCommonData::new(),
+			z_axis: AxisData::new(ZTickAxis),
+			contour_base: false,
+			contour_surface: false,
+			contour_auto: Auto,
+			contour_levels: None,
+			contour_style: Linear,
+			contour_label: Auto,
+		}
+	}
+
 	/// Draws a 3D surface from a rectangular array of data by connecting the individual datapoints with polygons.
 	///
 	/// #Arguments:
@@ -35,9 +49,12 @@ impl Axes3D
 	///                  By default this will be `(0, 0)` and `(num_rows - 1, num_cols - 1)`.
 	/// * `options` - Array of PlotOption controlling the appearance of the surface. Relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
-	pub fn surface<'l, T: DataType, X: IntoIterator<Item = T>>(&'l mut self, mat: X, num_rows: usize, num_cols: usize, dimensions: Option<(f64, f64, f64, f64)>, options: &[PlotOption]) -> &'l mut Axes3D
+	pub fn surface<'l, T: DataType, X: IntoIterator<Item = T>>(&'l mut self, mat: X, num_rows: usize, num_cols: usize, dimensions: Option<(f64, f64, f64, f64)>, options: &[PlotOption<'m>])
+		-> &'l mut Self
 	{
-		self.common.plot_matrix(Pm3D, true, mat, num_rows, num_cols, dimensions, options);
+		self.common
+			.elems
+			.push(PlotElement::new_plot_matrix(Pm3D, true, mat, num_rows, num_cols, dimensions, options.to_vec()));
 		self
 	}
 
@@ -46,14 +63,14 @@ impl Axes3D
 	/// #Arguments:
 	/// * `pitch` - Pitch, in degrees. Value of 0 is looking straight down on the XY plane, Z pointing out of the screen.
 	/// * `yaw` - Yaw, in degrees. Value of 0 is looking at the XZ plane, Y point into the screen.
-	pub fn set_view<'l>(&'l mut self, pitch: f64, yaw: f64) -> &'l mut Axes3D
+	pub fn set_view<'l>(&'l mut self, pitch: f64, yaw: f64) -> &'l mut Self
 	{
 		writeln!(&mut self.common.commands, "set view {:.12e},{:.12e}", pitch, yaw);
 		self
 	}
 
 	/// Sets the view to be a map. Useful for images and contour plots.
-	pub fn set_view_map<'l>(&'l mut self) -> &'l mut Axes3D
+	pub fn set_view_map<'l>(&'l mut self) -> &'l mut Self
 	{
 		writeln!(&mut self.common.commands, "set view map");
 		self
@@ -69,21 +86,23 @@ impl Axes3D
 	///      * `TextColor` - Specifies the color of the label
 	///      * `Rotate` - Specifies the rotation of the label
 	///      * `Align` - Specifies how to align the label
-	pub fn set_z_label<'l>(&'l mut self, text: &str, options: &[LabelOption]) -> &'l mut Axes3D
+	pub fn set_z_label<'l>(&'l mut self, text: &str, options: &[LabelOption]) -> &'l mut Self
 	{
 		self.get_common_data_mut().set_label_common(ZLabel, text, options);
 		self
 	}
 
 	/// Like `set_x_ticks` but for the Z axis.
-	pub fn set_z_ticks<'l>(&'l mut self, tick_placement: Option<(AutoOption<f64>, u32)>, tick_options: &[TickOption], label_options: &[LabelOption]) -> &'l mut Axes3D
+	pub fn set_z_ticks<'l>(&'l mut self, tick_placement: Option<(AutoOption<f64>, u32)>, tick_options: &[TickOption<'m>], label_options: &[LabelOption<'m>])
+		-> &'l mut Self
 	{
 		self.z_axis.set_ticks(tick_placement, tick_options, label_options);
 		self
 	}
 
 	/// Like `set_x_ticks_custom` but for the the Y axis.
-	pub fn set_z_ticks_custom<'l, T: DataType, TL: IntoIterator<Item = Tick<T>>>(&'l mut self, ticks: TL, tick_options: &[TickOption], label_options: &[LabelOption]) -> &'l mut Axes3D
+	pub fn set_z_ticks_custom<'l, T: DataType, TL: IntoIterator<Item = Tick<T>>>(&'l mut self, ticks: TL, tick_options: &[TickOption<'m>], label_options: &[LabelOption<'m>])
+		-> &'l mut Self
 	{
 		self.z_axis.set_ticks_custom(ticks, tick_options, label_options);
 		self
@@ -94,7 +113,7 @@ impl Axes3D
 	/// # Arguments
 	/// * `min` - Minimum Z value
 	/// * `max` - Maximum Z value
-	pub fn set_z_range<'l>(&'l mut self, min: AutoOption<f64>, max: AutoOption<f64>) -> &'l mut Axes3D
+	pub fn set_z_range<'l>(&'l mut self, min: AutoOption<f64>, max: AutoOption<f64>) -> &'l mut Self
 	{
 		self.z_axis.set_range(min, max);
 		self
@@ -112,7 +131,7 @@ impl Axes3D
 	///
 	/// # Arguments
 	/// * `base` - If Some, then specifies base of the logarithm, if None makes the axis not be logarithmic
-	pub fn set_z_log<'l>(&'l mut self, base: Option<f64>) -> &'l mut Axes3D
+	pub fn set_z_log<'l>(&'l mut self, base: Option<f64>) -> &'l mut Self
 	{
 		self.z_axis.set_log(base);
 		self
@@ -122,7 +141,7 @@ impl Axes3D
 	///
 	/// # Arguments
 	/// * `show` - Whether to show the grid.
-	pub fn set_z_grid<'l>(&'l mut self, show: bool) -> &'l mut Axes3D
+	pub fn set_z_grid<'l>(&'l mut self, show: bool) -> &'l mut Self
 	{
 		self.z_axis.set_grid(show);
 		self
@@ -138,7 +157,8 @@ impl Axes3D
 	///             otherwise an empty string disables the legend and labels.
 	/// * `levels` - Auto picks some default number of levels, otherwise you can pass a set nominal number instead. The number is nominal as
 	///              contours are placed at nice values of Z, and thus there may be fewer of them than this number.
-	pub fn show_contours<'l>(&'l mut self, base: bool, surface: bool, style: ContourStyle, label: AutoOption<&str>, levels: AutoOption<u32>) -> &'l mut Axes3D
+	pub fn show_contours<'l>(&'l mut self, base: bool, surface: bool, style: ContourStyle, label: AutoOption<&str>, levels: AutoOption<u32>)
+		-> &'l mut Self
 	{
 		self.contour_base = base;
 		self.contour_surface = surface;
@@ -158,9 +178,8 @@ impl Axes3D
 	/// * `label` - Auto sets the label automatically and enables the legend, Fix() allows you specify a format string (using C style formatting),
 	///             otherwise an empty string disables the legend and labels.
 	/// * `levels` - A set of levels.
-	pub fn show_contours_custom<'l, T: DataType,
-	                            TC: IntoIterator<Item = T>>(&'l mut self, base: bool, surface: bool,
-	                                             style: ContourStyle, label: AutoOption<&str>, levels: TC) -> &'l mut Axes3D
+	pub fn show_contours_custom<'l, T: DataType, TC: IntoIterator<Item = T>>(&'l mut self, base: bool, surface: bool, style: ContourStyle, label: AutoOption<&str>, levels: TC)
+		-> &'l mut Self
 	{
 		self.contour_base = base;
 		self.contour_surface = surface;
@@ -172,43 +191,27 @@ impl Axes3D
 	}
 }
 
-pub fn new_axes3d() -> Axes3D
+impl<'l> AxesCommonPrivate<'l> for Axes3D<'l>
 {
-	Axes3D
-	{
-		common: AxesCommonData::new(),
-		z_axis: AxisData::new(ZTickAxis),
-		contour_base: false,
-		contour_surface: false,
-		contour_auto: Auto,
-		contour_levels: None,
-		contour_style: Linear,
-		contour_label: Auto
-	}
-}
-
-impl AxesCommonPrivate for Axes3D
-{
-	fn get_common_data_mut<'l>(&'l mut self) -> &'l mut AxesCommonData
+	fn get_common_data_mut(&mut self) -> &mut AxesCommonData<'l>
 	{
 		&mut self.common
 	}
 
-	fn get_common_data<'l>(&'l self) -> &'l AxesCommonData
+	fn get_common_data(&self) -> &AxesCommonData<'l>
 	{
 		&self.common
 	}
 }
 
-impl AxesCommon for Axes3D {}
+impl<'l> AxesCommon<'l> for Axes3D<'l> {}
 
-#[doc(hidden)]
-pub trait Axes3DPrivate
+pub(crate) trait Axes3DPrivate
 {
 	fn write_out(&self, writer: &mut Writer);
 }
 
-impl Axes3DPrivate for Axes3D
+impl<'l> Axes3DPrivate for Axes3D<'l>
 {
 	fn write_out(&self, w: &mut Writer)
 	{
@@ -236,13 +239,17 @@ impl Axes3DPrivate for Axes3D
 		if self.contour_base || self.contour_surface
 		{
 			write!(w, "set contour ");
-			write!(w, "{}", match (self.contour_base, self.contour_surface)
-			{
-				(true, false) => "base",
-				(false, true) => "surface",
-				(true, true) => "both",
-				_ => unreachable!()
-			});
+			write!(
+				w,
+				"{}",
+				match (self.contour_base, self.contour_surface)
+				{
+					(true, false) => "base",
+					(false, true) => "surface",
+					(true, true) => "both",
+					_ => unreachable!(),
+				}
+			);
 			write!(w, "\n");
 
 			match self.contour_label
@@ -255,7 +262,7 @@ impl Axes3DPrivate for Axes3D
 				else
 				{
 					writeln!(w, r#"set clabel "{}""#, s)
-				}
+				},
 			};
 
 			fn set_cntrparam<F: FnOnce(&mut Writer)>(w: &mut Writer, wr: F)
@@ -265,47 +272,45 @@ impl Axes3DPrivate for Axes3D
 				write!(w, "\n");
 			}
 
-			set_cntrparam(w, |w|
-			{
-				write!(w, "{}", match self.contour_style
-				{
-					Linear => "linear ",
-					Cubic(..) => "cubicspline",
-					Spline(..) => "bspline",
-				});
+			set_cntrparam(w, |w| {
+				write!(
+					w,
+					"{}",
+					match self.contour_style
+					{
+						Linear => "linear ",
+						Cubic(..) => "cubicspline",
+						Spline(..) => "bspline",
+					}
+				);
 			});
 
-			set_cntrparam(w, |w|
-			{
+			set_cntrparam(w, |w| {
 				let pt = match self.contour_style
 				{
 					Cubic(pt) => Some(pt),
 					Spline(pt, _) => Some(pt),
-					_ => None
+					_ => None,
 				};
 
-				pt.map(|pt|
-				{
+				pt.map(|pt| {
 					write!(w, "points {}", clamp(pt, 2, 100));
 				});
 			});
 
-			set_cntrparam(w, |w|
-			{
+			set_cntrparam(w, |w| {
 				let ord = match self.contour_style
 				{
 					Spline(_, ord) => Some(ord),
-					_ => None
+					_ => None,
 				};
 
-				ord.map(|ord|
-				{
+				ord.map(|ord| {
 					write!(w, "order {}", clamp(ord, 2, 10));
 				});
 			});
 
-			set_cntrparam(w, |w|
-			{
+			set_cntrparam(w, |w| {
 				write!(w, "levels ");
 				match self.contour_levels
 				{
@@ -322,7 +327,7 @@ impl Axes3DPrivate for Axes3D
 							}
 							left -= 1;
 						}
-					},
+					}
 					None =>
 					{
 						match self.contour_auto
