@@ -2,40 +2,316 @@
 //
 // All rights reserved. Distributed under LGPL 3.0. For full terms see the file LICENSE.
 
-use std::io::Write;
-
-use datatype::*;
-use coordinates::*;
-use options::*;
-use writer::*;
-
-pub use self::LabelType::*;
-pub use self::TickAxis::*;
-pub use self::PlotType::*;
 use self::DataSourceType::*;
 
-impl PlotWriter for Vec<u8>
+pub use self::LabelType::*;
+pub use self::PlotType::*;
+pub use self::TickAxis::*;
+use coordinates::*;
+
+use datatype::*;
+use options::*;
+use std::io::Write;
+use writer::*;
+
+pub struct PlotElement<'l>
 {
-	fn write_data<T: DataType>(&mut self, v: T)
-	{
-		self.write_le_f64(v.get());
-	}
+	data: Vec<f64>,
+	num_rows: usize,
+	num_cols: usize,
+	plot_type: PlotType,
+	source_type: DataSourceType,
+	is_3d: bool,
+	options: Vec<PlotOption<'l>>,
 }
 
-pub struct PlotElement
+impl<'l> PlotElement<'l>
 {
-	pub args: Vec<u8>,
-	pub data: Vec<u8>
-}
-
-impl PlotElement
-{
-	pub fn new() -> PlotElement
+	pub fn new_plot2<T1, X1, T2, X2>(plot_type: PlotType, x1: X1, x2: X2, options: Vec<PlotOption<'l>>) -> PlotElement
+	where
+		T1: DataType,
+		X1: IntoIterator<Item = T1>,
+		T2: DataType,
+		X2: IntoIterator<Item = T2>,
 	{
-		PlotElement
+		let mut num_rows = 0;
+		let mut data = vec![];
+		// TODO: Reserve.
+		for (x1, x2) in x1.into_iter().zip(x2.into_iter())
 		{
-			args: vec![],
-			data: vec![],
+			data.push(x1.get());
+			data.push(x2.get());
+			num_rows += 1;
+		}
+
+		PlotElement {
+			data: data,
+			num_rows: num_rows,
+			num_cols: 2,
+			plot_type: plot_type,
+			source_type: Record,
+			is_3d: false,
+			options: options,
+		}
+	}
+
+	pub fn new_plot3<T1, X1, T2, X2, T3, X3>(plot_type: PlotType, x1: X1, x2: X2, x3: X3, options: Vec<PlotOption<'l>>) -> PlotElement
+	where
+		T1: DataType,
+		X1: IntoIterator<Item = T1>,
+		T2: DataType,
+		X2: IntoIterator<Item = T2>,
+		T3: DataType,
+		X3: IntoIterator<Item = T3>,
+	{
+		let mut num_rows = 0;
+		let mut data = vec![];
+		// TODO: Reserve.
+		for ((x1, x2), x3) in x1.into_iter().zip(x2.into_iter()).zip(x3.into_iter())
+		{
+			data.push(x1.get());
+			data.push(x2.get());
+			data.push(x3.get());
+			num_rows += 1;
+		}
+
+		PlotElement {
+			data: data,
+			num_rows: num_rows,
+			num_cols: 3,
+			plot_type: plot_type,
+			source_type: Record,
+			is_3d: false,
+			options: options,
+		}
+	}
+
+	pub fn new_plot_matrix<T: DataType, X: IntoIterator<Item = T>>(
+		plot_type: PlotType, is_3d: bool, mat: X, num_rows: usize, num_cols: usize, dimensions: Option<(f64, f64, f64, f64)>,
+		options: Vec<PlotOption<'l>>,
+	) -> PlotElement
+	{
+		let mut count = 0;
+		let mut data = vec![];
+		// TODO: Reserve.
+		for x in mat
+		{
+			data.push(x.get());
+			count += 1;
+		}
+
+		if count < num_rows * num_cols
+		{
+			for _ in 0..num_rows * num_cols - count
+			{
+				use std::f64;
+				data.push(f64::NAN);
+			}
+		}
+
+		let source_type = match dimensions
+		{
+			Some((x1, y1, x2, y2)) => SizedArray(x1, y1, x2, y2),
+			None => Array,
+		};
+
+		PlotElement {
+			data: data,
+			num_rows: num_rows,
+			num_cols: num_cols,
+			plot_type: plot_type,
+			source_type: source_type,
+			is_3d: is_3d,
+			options: options,
+		}
+	}
+
+	fn write_args(&self, writer: &mut Writer)
+	{
+		let options = &self.options;
+		match self.source_type
+		{
+			Record =>
+			{
+				write!(writer, r#" "-" binary endian=little record={} format="%float64" using "#, self.num_rows);
+
+				let mut col_idx = 1;
+				while col_idx < self.num_cols + 1
+				{
+					write!(writer, "{}", col_idx);
+					if col_idx < self.num_cols
+					{
+						writer.write_str(":");
+					}
+					col_idx += 1;
+				}
+			}
+			_ =>
+			{
+				write!(
+					writer,
+					r#" "-" binary endian=little array=({},{}) format="%float64" "#,
+					self.num_cols,
+					self.num_rows
+				);
+
+				match self.source_type
+				{
+					SizedArray(x1, y1, x2, y2) =>
+					{
+						let (x1, x2) = if x1 > x2
+						{
+							(x2, x1)
+						}
+						else
+						{
+							(x1, x2)
+						};
+
+						let (y1, y2) = if y1 > y2
+						{
+							(y2, y1)
+						}
+						else
+						{
+							(y1, y2)
+						};
+						write!(writer, "origin=({:.12e},{:.12e}", x1, y1);
+						if self.is_3d
+						{
+							write!(writer, ",0");
+						}
+						write!(writer, ") ");
+						if self.num_cols > 1
+						{
+							write!(writer, "dx={:.12e} ", (x2 - x1) / (self.num_cols as f64 - 1.0));
+						}
+						else
+						{
+							write!(writer, "dx=1 ");
+						}
+						if self.num_rows > 1
+						{
+							write!(writer, "dy={:.12e} ", (y2 - y1) / (self.num_rows as f64 - 1.0));
+						}
+						else
+						{
+							write!(writer, "dy=1 ");
+						}
+					}
+					_ => (),
+				}
+			}
+		}
+
+		writer.write_str(" with ");
+		let type_str = match self.plot_type
+		{
+			Lines => "lines",
+			Points => "points",
+			LinesPoints => "linespoints",
+			XErrorLines => "xerrorlines",
+			YErrorLines => "yerrorlines",
+			XErrorBars => "xerrorbars",
+			YErrorBars => "yerrorbars",
+			FillBetween => "filledcurves",
+			Boxes => "boxes",
+			Pm3D => "pm3d",
+			Image => "image",
+		};
+		writer.write_str(type_str);
+
+		if self.plot_type.is_fill()
+		{
+			match self.plot_type
+			{
+				FillBetween =>
+				{
+					let mut found = false;
+					first_opt!{options,
+						FillRegion(d) =>
+						{
+							found = true;
+							writer.write_str(match d
+							{
+								Above => " above",
+								Below => " below",
+								Between => " closed",
+							});
+						}
+					}
+					if !found
+					{
+						writer.write_str(" closed");
+					}
+				}
+				_ => (),
+			}
+
+			writer.write_str(" fill transparent solid ");
+
+			first_opt!{self.options,
+				FillAlpha(a) =>
+				{
+					write!(writer, "{:.12e}", a);
+				}
+			}
+
+			if self.plot_type.is_line()
+			{
+				writer.write_str(" border");
+				first_opt!{self.options,
+					BorderColor(s) =>
+					{
+						write!(writer, r#" rgb "{}""#, s);
+					}
+				}
+			}
+			else
+			{
+				writer.write_str(" noborder");
+			}
+		}
+
+		if self.plot_type.is_line()
+		{
+			AxesCommonData::write_line_options(writer, options);
+		}
+
+		if self.plot_type.is_points()
+		{
+			first_opt!{self.options,
+				PointSymbol(s) =>
+				{
+					write!(writer, " pt {}", char_to_symbol(s));
+				}
+			}
+
+			first_opt!{self.options,
+				PointSize(z) =>
+				{
+					write!(writer, " ps {}", z);
+				}
+			}
+		}
+
+		AxesCommonData::write_color_options(writer, &self.options, None);
+
+		writer.write_str(" t \"");
+		first_opt!{self.options,
+			Caption(s) =>
+			{
+				writer.write_str(s);
+			}
+		}
+		writer.write_str("\"");
+	}
+
+	fn write_data(&self, writer: &mut Writer)
+	{
+		for d in &self.data
+		{
+			writer.write_le_f64(*d);
 		}
 	}
 }
@@ -59,22 +335,21 @@ impl LabelType
 		match *self
 		{
 			Label(..) => true,
-			_ => false
+			_ => false,
 		}
 	}
 }
 
-pub fn write_out_label_options<T: PlotWriter + Writer>(label_type: LabelType, options: &[LabelOption], writer: &mut T)
+pub fn write_out_label_options(label_type: LabelType, options: &[LabelOption], writer: &mut Writer)
 {
-	let w = writer as &mut Writer;
-
+	let w = writer;
 	match label_type
 	{
 		Label(x, y) =>
 		{
 			write!(w, " at {},{} front", x, y);
 		}
-		_ => ()
+		_ => (),
 	}
 
 	first_opt!{options,
@@ -199,6 +474,8 @@ pub enum PlotType
 	LinesPoints,
 	XErrorLines,
 	YErrorLines,
+	XErrorBars,
+	YErrorBars,
 	FillBetween,
 	Boxes,
 	Pm3D,
@@ -211,12 +488,8 @@ impl PlotType
 	{
 		match *self
 		{
-			Lines |
-			LinesPoints |
-			XErrorLines |
-			Boxes |
-			YErrorLines => true,
-			_ => false
+			Lines | LinesPoints | XErrorLines | Boxes | YErrorLines => true,
+			_ => false,
 		}
 	}
 
@@ -224,11 +497,8 @@ impl PlotType
 	{
 		match *self
 		{
-			Points |
-			LinesPoints |
-			XErrorLines |
-			YErrorLines => true,
-			_ => false
+			Points | LinesPoints | XErrorLines | YErrorLines | XErrorBars | YErrorBars => true,
+			_ => false,
 		}
 	}
 
@@ -236,18 +506,25 @@ impl PlotType
 	{
 		match *self
 		{
-			Boxes |
-			FillBetween => true,
-			_ => false
+			Boxes | FillBetween => true,
+			_ => false,
 		}
 	}
 }
 
-pub struct AxisData
+pub enum TickType
 {
-	pub ticks_buf: Vec<u8>,
+	None,
+	Custom(Vec<Tick<f64>>),
+	Auto(AutoOption<f64>, u32),
+}
+
+pub struct AxisData<'l>
+{
+	pub tick_options: Vec<TickOption<'l>>,
+	pub label_options: Vec<LabelOption<'l>>,
+	pub tick_type: TickType,
 	pub log_base: Option<f64>,
-	pub mticks: i32,
 	pub axis: TickAxis,
 	pub min: AutoOption<f64>,
 	pub max: AutoOption<f64>,
@@ -255,15 +532,15 @@ pub struct AxisData
 	pub grid: bool,
 }
 
-impl AxisData
+impl<'m> AxisData<'m>
 {
-	pub fn new(axis: TickAxis) -> AxisData
+	pub fn new(axis: TickAxis) -> Self
 	{
-		AxisData
-		{
-			ticks_buf: vec![],
+		AxisData {
+			tick_options: vec![],
+			label_options: vec![],
+			tick_type: TickType::Auto(Auto, 0),
 			log_base: None,
-			mticks: 0,
 			axis: axis,
 			min: Auto,
 			max: Auto,
@@ -282,7 +559,7 @@ impl AxisData
 				w.write_str(self.axis.to_axis_str());
 				write!(w, " {:.12e}", base);
 				true
-			},
+			}
 			None =>
 			{
 				w.write_str("unset logscale ");
@@ -290,39 +567,42 @@ impl AxisData
 				false
 			}
 		};
-
 		w.write_str("\n");
-		if self.mticks > 0
+
+		match self.tick_type
 		{
-			write!(w, "set m{} ", self.axis.to_tick_str());
-			if log
+			TickType::Auto(_, mticks) =>
 			{
-				writeln!(w, "default");
+				write!(w, "set m{} ", self.axis.to_tick_str());
+				if log
+				{
+					writeln!(w, "default");
+				}
+				else
+				{
+					writeln!(w, "{}", mticks as i32 + 1);
+				}
 			}
-			else
+			_ =>
 			{
-				writeln!(w, "{}", self.mticks as i32 + 1);
+				writeln!(w, "unset m{}", self.axis.to_tick_str());
 			}
 		}
-		else
-		{
-			writeln!(w, "unset m{}", self.axis.to_tick_str());
-		}
-
 		w.write_str("\n");
+
 		w.write_str("set ");
 		w.write_str(self.axis.to_range_str());
 		w.write_str(" [");
 		match self.min
 		{
 			Fix(v) => write!(w, "{:.12e}", v),
-			Auto => w.write_str("*")
+			Auto => w.write_str("*"),
 		};
 		w.write_str(":");
 		match self.max
 		{
 			Fix(v) => write!(w, "{:.12e}", v),
-			Auto => w.write_str("*")
+			Auto => w.write_str("*"),
 		};
 		if self.reverse
 		{
@@ -333,170 +613,174 @@ impl AxisData
 			w.write_str("]\n");
 		}
 
-		w.write_all(&self.ticks_buf[..]);
-	}
-
-	pub fn set_ticks_custom<T: DataType, TL: IntoIterator<Item = Tick<T>>>(&mut self, ticks: TL, tick_options: &[TickOption], label_options: &[LabelOption])
-	{
-		// Set to 0 so that we don't get any non-custom ticks
-		self.mticks = 0;
+		let mut write_tick_options = true;
+		match self.tick_type
 		{
-			let c = &mut self.ticks_buf;
-			c.truncate(0);
-
-			c.write_str("set ");
-			c.write_str(self.axis.to_tick_str());
-			c.write_str(" (");
-
-			let mut first = true;
-			for tick in ticks
+			TickType::None =>
 			{
-				if first
-				{
-					first = false;
-				}
-				else
-				{
-					c.write_str(",");
-				}
+				write!(w, "unset {0}", self.axis.to_tick_str());
+				write_tick_options = false;
+			}
+			TickType::Auto(incr, _) =>
+			{
+				w.write_str("set ");
+				w.write_str(self.axis.to_tick_str());
 
-				let a = Auto;
-				let (ref pos, ref label, level) = match tick
+				match incr
 				{
-					Minor(ref pos) =>
+					Auto =>
 					{
-						(pos, &a, 1)
-					},
-					Major(ref pos, ref label) =>
-					{
-						(pos, label, 0)
+						w.write_str(" autofreq");
 					}
-				};
-
-				match **label
-				{
-					Fix(ref label) =>
+					Fix(incr) =>
 					{
-						c.write_str("\"");
-						c.write_str(&label[..]);
-						c.write_str("\" ");
-					},
-					Auto => ()
-				}
-				write!(&mut *c, "{:.12e} {}", pos.get(), level);
-			}
-			c.write_str(")");
-		}
-		self.set_ticks_options(tick_options, label_options);
-		self.ticks_buf.write_str("\n");
-	}
-
-	fn set_ticks_options(&mut self, tick_options: &[TickOption], label_options: &[LabelOption])
-	{
-		let c = &mut self.ticks_buf;
-		write_out_label_options(AxesTicks, label_options, c);
-
-		first_opt!{tick_options,
-			OnAxis(b) =>
-			{
-				c.write_str(match b
-				{
-					true => " axis",
-					false => " border",
-				});
-			}
-		}
-
-		first_opt!{tick_options,
-			Mirror(b) =>
-			{
-				c.write_str(match b
-				{
-					true => " mirror",
-					false => " nomirror",
-				});
-			}
-		}
-
-		first_opt!{tick_options,
-			Inward(b) =>
-			{
-				c.write_str(match b
-				{
-					true => " in",
-					false => " out",
-				});
-			}
-		}
-
-		let mut minor_scale = 0.5;
-		let mut major_scale = 0.5;
-
-		first_opt!{tick_options,
-			MinorScale(s) =>
-			{
-				minor_scale = s;
-			}
-		}
-
-		first_opt!{tick_options,
-			MajorScale(s) =>
-			{
-				major_scale = s;
-			}
-		}
-
-		write!(&mut *c, " scale {:.12e},{:.12e}", minor_scale, major_scale);
-
-		first_opt!{tick_options,
-			Format(f) =>
-			{
-				write!(&mut *c, r#" format "{}""#, f);
-			}
-		}
-	}
-
-	pub fn set_ticks(&mut self, tick_placement: Option<(AutoOption<f64>, u32)>, tick_options: &[TickOption], label_options: &[LabelOption])
-	{
-		self.ticks_buf.truncate(0);
-
-		self.mticks = match tick_placement
-		{
-			Some((incr, mticks)) =>
-			{
-				{
-					let c = &mut self.ticks_buf;
-					c.write_str("set ");
-					c.write_str(self.axis.to_tick_str());
-
-					match incr
-					{
-						Auto =>
+						if incr <= 0.0
 						{
-							c.write_str(" autofreq");
-						},
-						Fix(incr) =>
-						{
-							if incr <= 0.0
-							{
-								panic!("'incr' must be positive, but is actually {}", incr);
-							}
-							c.write_str(" ");
-							write!(&mut *c, " {:.12e}", incr);
+							panic!("'incr' must be positive, but is actually {}", incr);
 						}
+						w.write_str(" ");
+						write!(w, " {:.12e}", incr);
 					}
 				}
-
-				self.set_ticks_options(tick_options, label_options);
-				mticks as i32
-			},
-			None =>
-			{
-				write!(&mut self.ticks_buf, "unset {0}", self.axis.to_tick_str());
-				0
 			}
-		};
-		self.ticks_buf.write_str("\n");
+			TickType::Custom(ref ticks) =>
+			{
+				w.write_str("set ");
+				w.write_str(self.axis.to_tick_str());
+				w.write_str(" (");
+
+				let mut first = true;
+				for tick in ticks
+				{
+					if first
+					{
+						first = false;
+					}
+					else
+					{
+						w.write_str(",");
+					}
+
+					let a = Auto;
+					let (ref pos, ref label, level) = match *tick
+					{
+						Minor(ref pos) => (pos, &a, 1),
+						Major(ref pos, ref label) => (pos, label, 0),
+					};
+
+					match **label
+					{
+						Fix(ref label) =>
+						{
+							w.write_str("\"");
+							w.write_str(&label[..]);
+							w.write_str("\" ");
+						}
+						Auto => (),
+					}
+					write!(w, "{:.12e} {}", pos.get(), level);
+				}
+				w.write_str(")");
+			}
+		}
+
+		if write_tick_options
+		{
+			let label_options = &self.label_options;
+			let tick_options = &self.tick_options;
+
+			write_out_label_options(AxesTicks, label_options, &mut *w);
+
+			first_opt!{tick_options,
+				OnAxis(b) =>
+				{
+					w.write_str(match b
+					{
+						true => " axis",
+						false => " border",
+					});
+				}
+			}
+
+			first_opt!{tick_options,
+				Mirror(b) =>
+				{
+					w.write_str(match b
+					{
+						true => " mirror",
+						false => " nomirror",
+					});
+				}
+			}
+
+			first_opt!{tick_options,
+				Inward(b) =>
+				{
+					w.write_str(match b
+					{
+						true => " in",
+						false => " out",
+					});
+				}
+			}
+
+			let mut minor_scale = 0.5;
+			let mut major_scale = 0.5;
+
+			first_opt!{tick_options,
+				MinorScale(s) =>
+				{
+					minor_scale = s;
+				}
+			}
+
+			first_opt!{tick_options,
+				MajorScale(s) =>
+				{
+					major_scale = s;
+				}
+			}
+
+			write!(w, " scale {:.12e},{:.12e}", minor_scale, major_scale);
+
+			first_opt!{tick_options,
+				Format(f) =>
+				{
+					write!(w, r#" format "{}""#, f);
+				}
+			}
+		}
+		w.write_str("\n");
+	}
+
+	pub fn set_ticks_custom<T: DataType, TL: IntoIterator<Item = Tick<T>>>(&mut self, ticks: TL, tick_options: &[TickOption<'m>], label_options: &[LabelOption<'m>])
+	{
+		self.tick_type = TickType::Custom(
+			ticks
+				.into_iter()
+				.map(|t| match t
+				{
+					Major(t, l) => Major(t.get(), l),
+					Minor(t) => Minor(t.get()),
+				})
+				.collect(),
+		);
+		self.tick_options = tick_options.into();
+		self.label_options = label_options.into();
+	}
+
+	pub fn set_ticks(&mut self, tick_placement: Option<(AutoOption<f64>, u32)>, tick_options: &[TickOption<'m>], label_options: &[LabelOption<'m>])
+	{
+		if let Some((incr, mticks)) = tick_placement
+		{
+			self.tick_type = TickType::Auto(incr, mticks);
+			self.tick_options = tick_options.into();
+			self.label_options = label_options.into();
+		}
+		else
+		{
+			self.tick_type = TickType::None
+		}
 	}
 
 	pub fn set_range(&mut self, min: AutoOption<f64>, max: AutoOption<f64>)
@@ -521,19 +805,6 @@ impl AxisData
 	}
 }
 
-pub struct AxesCommonData
-{
-	pub commands: Vec<u8>,
-	pub grid_options: Vec<u8>,
-	pub elems: Vec<PlotElement>,
-	pub grid_rows: u32,
-	pub grid_cols: u32,
-	pub grid_pos: Option<u32>,
-	pub x_axis: AxisData,
-	pub y_axis: AxisData,
-	pub cb_axis: AxisData,
-}
-
 pub fn char_to_symbol(c: char) -> i32
 {
 	match c
@@ -552,7 +823,7 @@ pub fn char_to_symbol(c: char) -> i32
 		'D' => 11,
 		'r' => 12,
 		'R' => 13,
-		a => panic!("Invalid symbol {}", a)
+		a => panic!("Invalid symbol {}", a),
 	}
 }
 
@@ -563,14 +834,28 @@ enum DataSourceType
 	SizedArray(f64, f64, f64, f64),
 }
 
-impl AxesCommonData
+pub struct AxesCommonData<'l>
 {
-	pub fn new() -> AxesCommonData
+	pub commands: Vec<u8>,
+	pub grid_options: Vec<PlotOption<'l>>,
+	pub grid_front: bool,
+	pub elems: Vec<PlotElement<'l>>,
+	pub grid_rows: u32,
+	pub grid_cols: u32,
+	pub grid_pos: Option<u32>,
+	pub x_axis: AxisData<'l>,
+	pub y_axis: AxisData<'l>,
+	pub cb_axis: AxisData<'l>,
+}
+
+impl<'m> AxesCommonData<'m>
+{
+	pub fn new() -> AxesCommonData<'m>
 	{
-		AxesCommonData
-		{
+		AxesCommonData {
 			commands: vec![],
 			grid_options: vec![],
+			grid_front: false,
 			elems: Vec::new(),
 			grid_rows: 0,
 			grid_cols: 0,
@@ -591,7 +876,18 @@ impl AxesCommonData
 				c.write_str(axis.to_tick_str());
 				c.write_str(" ");
 			}
-			c.write_all(&self.grid_options);
+
+			if self.grid_front
+			{
+				c.write_str("front ");
+			}
+			else
+			{
+				c.write_str("back ");
+			}
+
+			AxesCommonData::write_line_options(c, &self.grid_options);
+			AxesCommonData::write_color_options(c, &self.grid_options, None);
 			c.write_str("\n");
 		}
 	}
@@ -641,261 +937,11 @@ impl AxesCommonData
 			Some(s) =>
 			{
 				write!(c, r#" lc rgb "{}""#, s);
-			},
-			None => ()
+			}
+			None => (),
 		}
 	}
 
-	pub fn plot2<T1: DataType, X1: IntoIterator<Item = T1>,
-	             T2: DataType, X2: IntoIterator<Item = T2>>(&mut self, plot_type: PlotType, x1: X1, x2: X2, options: &[PlotOption])
-	{
-		let l = self.elems.len();
-		self.elems.push(PlotElement::new());
-		let mut num_rows = 0;
-
-		{
-			let data = &mut self.elems[l].data;
-			for (x1, x2) in x1.into_iter().zip(x2.into_iter())
-			{
-				data.write_data(x1);
-				data.write_data(x2);
-				num_rows += 1;
-			}
-		}
-
-		self.write_common_commands(l, num_rows, 2, plot_type, Record, false, options);
-	}
-
-	pub fn plot3<T1: DataType, X1: IntoIterator<Item = T1>,
-			     T2: DataType, X2: IntoIterator<Item = T2>,
-			     T3: DataType, X3: IntoIterator<Item = T3>>(&mut self, plot_type: PlotType, x1: X1, x2: X2, x3: X3, options: &[PlotOption])
-	{
-		let l = self.elems.len();
-		self.elems.push(PlotElement::new());
-		let mut num_rows = 0;
-
-		{
-			let data = &mut self.elems[l].data;
-			for ((x1, x2), x3) in x1.into_iter().zip(x2.into_iter()).zip(x3.into_iter())
-			{
-				data.write_data(x1);
-				data.write_data(x2);
-				data.write_data(x3);
-				num_rows += 1;
-			}
-		}
-
-		self.write_common_commands(l, num_rows, 3, plot_type, Record, false, options);
-	}
-
-	pub fn plot_matrix<T: DataType, X: IntoIterator<Item = T>>(&mut self, plot_type: PlotType, is_3d: bool, mat: X, num_rows: usize, num_cols: usize,
-	                                                dimensions: Option<(f64, f64, f64, f64)>, options: &[PlotOption])
-	{
-		let l = self.elems.len();
-		self.elems.push(PlotElement::new());
-
-		{
-			let mut count = 0;
-			let data = &mut self.elems[l].data;
-			for x in mat
-			{
-				data.write_data(x);
-				count += 1;
-			}
-
-			if count < num_rows * num_cols
-			{
-				for _ in 0..num_rows * num_cols - count
-				{
-					use std::f64;
-					data.write_data(f64::NAN);
-				}
-			}
-		}
-
-		let source_type = match dimensions
-		{
-			Some((x1, y1, x2, y2)) => SizedArray(x1, y1, x2, y2),
-			None => Array
-		};
-		self.write_common_commands(l, num_rows, num_cols, plot_type, source_type, is_3d, options);
-	}
-
-	fn write_common_commands(&mut self, elem_idx: usize, num_rows: usize, num_cols: usize, plot_type: PlotType,
-	                         source_type: DataSourceType, is_3d: bool, options: &[PlotOption])
-	{
-		let args = &mut self.elems[elem_idx].args as &mut Writer;
-		match source_type
-		{
-			Record =>
-			{
-				write!(args, r#" "-" binary endian=little record={} format="%float64" using "#, num_rows);
-
-				let mut col_idx = 1;
-				while col_idx < num_cols + 1
-				{
-					write!(args, "{}", col_idx);
-					if col_idx < num_cols
-					{
-						args.write_str(":");
-					}
-					col_idx += 1;
-				}
-			},
-			_ =>
-			{
-				write!(args, r#" "-" binary endian=little array=({},{}) format="%float64" "#, num_cols, num_rows);
-
-				match source_type
-				{
-					SizedArray(x1, y1, x2, y2) =>
-					{
-						let (x1, x2) = if x1 > x2
-						{
-							(x2, x1)
-						}
-						else
-						{
-							(x1, x2)
-						};
-
-						let (y1, y2) = if y1 > y2
-						{
-							(y2, y1)
-						}
-						else
-						{
-							(y1, y2)
-						};
-						write!(args, "origin=({:.12e},{:.12e}", x1, y1);
-						if is_3d
-						{
-							write!(args, ",0");
-						}
-						write!(args, ") ");
-						if num_cols > 1
-						{
-							write!(args, "dx={:.12e} ", (x2 - x1) / (num_cols as f64 - 1.0));
-						}
-						else
-						{
-							write!(args, "dx=1 ");
-						}
-						if num_rows > 1
-						{
-							write!(args, "dy={:.12e} ", (y2 - y1) / (num_rows as f64 - 1.0));
-						}
-						else
-						{
-							write!(args, "dy=1 ");
-						}
-					},
-					_ => ()
-				}
-			}
-		}
-
-		args.write_str(" with ");
-		let type_str = match plot_type
-		{
-			Lines => "lines",
-			Points => "points",
-			LinesPoints => "linespoints",
-			XErrorLines => "xerrorlines",
-			YErrorLines => "yerrorlines",
-			FillBetween => "filledcurves",
-			Boxes => "boxes",
-			Pm3D => "pm3d",
-			Image => "image",
-		};
-		args.write_str(type_str);
-
-		if plot_type.is_fill()
-		{
-			match plot_type
-			{
-				FillBetween =>
-				{
-					let mut found = false;
-					first_opt!{options,
-						FillRegion(d) =>
-						{
-							found = true;
-							args.write_str(match d
-							{
-								Above => " above",
-								Below => " below",
-								Between => " closed",
-							});
-						}
-					}
-					if !found
-					{
-						args.write_str(" closed");
-					}
-				},
-				_ => ()
-			}
-
-			args.write_str(" fill transparent solid ");
-
-			first_opt!{options,
-				FillAlpha(a) =>
-				{
-					write!(args, "{:.12e}", a);
-				}
-			}
-
-			if plot_type.is_line()
-			{
-				args.write_str(" border");
-				first_opt!{options,
-					BorderColor(s) =>
-					{
-						write!(args, r#" rgb "{}""#, s);
-					}
-				}
-			}
-			else
-			{
-				args.write_str(" noborder");
-			}
-		}
-
-		if plot_type.is_line()
-		{
-			AxesCommonData::write_line_options(args, options);
-		}
-
-		if plot_type.is_points()
-		{
-			first_opt!{options,
-				PointSymbol(s) =>
-				{
-					write!(args, " pt {}", char_to_symbol(s));
-				}
-			}
-
-			first_opt!{options,
-				PointSize(z) =>
-				{
-					write!(args, " ps {}", z);
-				}
-			}
-		}
-
-		AxesCommonData::write_color_options(args, options, None);
-
-		args.write_str(" t \"");
-		first_opt!{options,
-			Caption(s) =>
-			{
-				args.write_str(s);
-			}
-		}
-		args.write_str("\"");
-	}
-
 	pub fn write_out_commands(&self, writer: &mut Writer)
 	{
 		writer.write_all(&self.commands[..]);
@@ -915,7 +961,7 @@ impl AxesCommonData
 			{
 				write!(writer, ",");
 			}
-			writer.write_all(&e.args[..]);
+			e.write_args(writer);
 			first = false;
 		}
 
@@ -923,7 +969,7 @@ impl AxesCommonData
 
 		for e in self.elems.iter()
 		{
-			writer.write_all(&e.data[..]);
+			e.write_data(writer);
 		}
 	}
 
@@ -941,7 +987,7 @@ impl AxesCommonData
 			CBLabel => "cblabe",
 			TitleLabel => "title",
 			Label(..) => "label",
-			_ => panic!("Invalid label type")
+			_ => panic!("Invalid label type"),
 		};
 		c.write_str(label_str);
 
@@ -949,22 +995,20 @@ impl AxesCommonData
 		c.write_str(text);
 		c.write_str("\"");
 
-		write_out_label_options(label_type, options, c);
+		write_out_label_options(label_type, options, &mut *c);
 
 		c.write_str("\n");
 	}
-
-
 }
 
 #[doc(hidden)]
-pub trait AxesCommonPrivate
+pub trait AxesCommonPrivate<'l>
 {
-	fn get_common_data<'l>(&'l self) -> &'l AxesCommonData;
-	fn get_common_data_mut<'l>(&'l mut self) -> &'l mut AxesCommonData;
+	fn get_common_data(&self) -> &AxesCommonData<'l>;
+	fn get_common_data_mut(&mut self) -> &mut AxesCommonData<'l>;
 }
 
-pub trait AxesCommon : AxesCommonPrivate
+pub trait AxesCommon<'m>: AxesCommonPrivate<'m>
 {
 	/// Set the position of the axes on the figure using grid coordinates.
 	/// # Arguments
@@ -1021,7 +1065,7 @@ pub trait AxesCommon : AxesCommonPrivate
 				Fix(r) =>
 				{
 					writeln!(c, "set size ratio {:.12e}", r);
-				},
+				}
 				Auto =>
 				{
 					writeln!(c, "set size noratio");
@@ -1040,21 +1084,21 @@ pub trait AxesCommon : AxesCommonPrivate
 	///      * `TextColor` - Specifies the color of the label
 	///      * `Rotate` - Specifies the rotation of the label
 	///      * `Align` - Specifies how to align the label
-	fn set_x_label<'l>(&'l mut self, text: &str, options: &[LabelOption]) -> &'l mut Self
+	fn set_x_label<'l>(&'l mut self, text: &str, options: &[LabelOption<'m>]) -> &'l mut Self
 	{
 		self.get_common_data_mut().set_label_common(XLabel, text, options);
 		self
 	}
 
 	/// Like `set_x_label`, but for the Y axis
-	fn set_y_label<'l>(&'l mut self, text: &str, options: &[LabelOption]) -> &'l mut Self
+	fn set_y_label<'l>(&'l mut self, text: &str, options: &[LabelOption<'m>]) -> &'l mut Self
 	{
 		self.get_common_data_mut().set_label_common(YLabel, text, options);
 		self
 	}
 
 	/// Like `set_x_label`, but for the color bar
-	fn set_cb_label<'l>(&'l mut self, text: &str, options: &[LabelOption]) -> &'l mut Self
+	fn set_cb_label<'l>(&'l mut self, text: &str, options: &[LabelOption<'m>]) -> &'l mut Self
 	{
 		self.get_common_data_mut().set_label_common(CBLabel, text, options);
 		self
@@ -1063,13 +1107,13 @@ pub trait AxesCommon : AxesCommonPrivate
 	/// Set the title for the axes
 	/// # Arguments
 	/// * `text` - Text of the title. Pass an empty string to hide the title
-	/// * `options` - Array of LabelOption controlling the appearance of the title. Relevant options are:
+	/// * `options` - Array of LabelOption<'m> controlling the appearance of the title. Relevant options are:
 	///      * `Offset` - Specifies the offset of the label
 	///      * `Font` - Specifies the font of the label
 	///      * `TextColor` - Specifies the color of the label
 	///      * `Rotate` - Specifies the rotation of the label
 	///      * `Align` - Specifies how to align the label
-	fn set_title<'l>(&'l mut self, text: &str, options: &[LabelOption]) -> &'l mut Self
+	fn set_title<'l>(&'l mut self, text: &str, options: &[LabelOption<'m>]) -> &'l mut Self
 	{
 		self.get_common_data_mut().set_label_common(TitleLabel, text, options);
 		self
@@ -1080,7 +1124,7 @@ pub trait AxesCommon : AxesCommonPrivate
 	/// * `text` - Text of the label
 	/// * `x` - X coordinate of the label
 	/// * `y` - Y coordinate of the label
-	/// * `options` - Array of LabelOption controlling the appearance of the label. Relevant options are:
+	/// * `options` - Array of LabelOption<'m> controlling the appearance of the label. Relevant options are:
 	///      * `Offset` - Specifies the offset of the label
 	///      * `Font` - Specifies the font of the label
 	///      * `TextColor` - Specifies the color of the label
@@ -1089,7 +1133,7 @@ pub trait AxesCommon : AxesCommonPrivate
 	///      * `MarkerSymbol` - Specifies the symbol for the marker. Omit to hide the marker
 	///      * `MarkerSize` - Specifies the size for the marker
 	///      * `MarkerColor` - Specifies the color for the marker
-	fn label<'l>(&'l mut self, text: &str, x: Coordinate, y: Coordinate, options: &[LabelOption]) -> &'l mut Self
+	fn label<'l>(&'l mut self, text: &str, x: Coordinate, y: Coordinate, options: &[LabelOption<'m>]) -> &'l mut Self
 	{
 		self.get_common_data_mut().set_label_common(Label(x, y), text, options);
 		self
@@ -1103,27 +1147,30 @@ pub trait AxesCommon : AxesCommonPrivate
 	///                      tuple value specifies the number of minor ticks. For logarithmic axes, non-zero values mean that the number of ticks usually
 	///                      equals to `ceil(log_base) - 2`.
 	/// * `tick_options` - Array of TickOption controlling the appearance of the ticks
-	/// * `label_options` - Array of LabelOption controlling the appearance of the tick labels. Relevant options are:
+	/// * `label_options` - Array of LabelOption<'m> controlling the appearance of the tick labels. Relevant options are:
 	///      * `Offset` - Specifies the offset of the label
 	///      * `Font` - Specifies the font of the label
 	///      * `TextColor` - Specifies the color of the label
 	///      * `Rotate` - Specifies the rotation of the label
 	///      * `Align` - Specifies how to align the label
-	fn set_x_ticks<'l>(&'l mut self, tick_placement: Option<(AutoOption<f64>, u32)>, tick_options: &[TickOption], label_options: &[LabelOption]) -> &'l mut Self
+	fn set_x_ticks<'l>(&'l mut self, tick_placement: Option<(AutoOption<f64>, u32)>, tick_options: &[TickOption<'m>], label_options: &[LabelOption<'m>])
+		-> &'l mut Self
 	{
 		self.get_common_data_mut().x_axis.set_ticks(tick_placement, tick_options, label_options);
 		self
 	}
 
 	/// Like `set_x_ticks` but for the Y axis.
-	fn set_y_ticks<'l>(&'l mut self, tick_placement: Option<(AutoOption<f64>, u32)>, tick_options: &[TickOption], label_options: &[LabelOption]) -> &'l mut Self
+	fn set_y_ticks<'l>(&'l mut self, tick_placement: Option<(AutoOption<f64>, u32)>, tick_options: &[TickOption<'m>], label_options: &[LabelOption<'m>])
+		-> &'l mut Self
 	{
 		self.get_common_data_mut().y_axis.set_ticks(tick_placement, tick_options, label_options);
 		self
 	}
 
 	/// Like `set_x_ticks` but for the color bar axis.
-	fn set_cb_ticks<'l>(&'l mut self, tick_placement: Option<(AutoOption<f64>, u32)>, tick_options: &[TickOption], label_options: &[LabelOption]) -> &'l mut Self
+	fn set_cb_ticks<'l>(&'l mut self, tick_placement: Option<(AutoOption<f64>, u32)>, tick_options: &[TickOption<'m>], label_options: &[LabelOption<'m>])
+		-> &'l mut Self
 	{
 		self.get_common_data_mut().cb_axis.set_ticks(tick_placement, tick_options, label_options);
 		self
@@ -1137,27 +1184,30 @@ pub trait AxesCommon : AxesCommonPrivate
 	///     The label can contain a single C printf style floating point formatting specifier which will be replaced by the
 	///     location of the tic.
 	/// * `tick_options` - Array of TickOption controlling the appearance of the ticks
-	/// * `label_options` - Array of LabelOption controlling the appearance of the tick labels. Relevant options are:
+	/// * `label_options` - Array of LabelOption<'m> controlling the appearance of the tick labels. Relevant options are:
 	///      * `Offset` - Specifies the offset of the label
 	///      * `Font` - Specifies the font of the label
 	///      * `TextColor` - Specifies the color of the label
 	///      * `Rotate` - Specifies the rotation of the label
 	///      * `Align` - Specifies how to align the label
-	fn set_x_ticks_custom<'l, T: DataType, TL: IntoIterator<Item = Tick<T>>>(&'l mut self, ticks: TL, tick_options: &[TickOption], label_options: &[LabelOption]) -> &'l mut Self
+	fn set_x_ticks_custom<'l, T: DataType, TL: IntoIterator<Item = Tick<T>>>(&'l mut self, ticks: TL, tick_options: &[TickOption<'m>], label_options: &[LabelOption<'m>])
+		-> &'l mut Self
 	{
 		self.get_common_data_mut().x_axis.set_ticks_custom(ticks, tick_options, label_options);
 		self
 	}
 
 	/// Like `set_x_ticks_custom` but for the the Y axis.
-	fn set_y_ticks_custom<'l, T: DataType, TL: IntoIterator<Item = Tick<T>>>(&'l mut self, ticks: TL, tick_options: &[TickOption], label_options: &[LabelOption]) -> &'l mut Self
+	fn set_y_ticks_custom<'l, T: DataType, TL: IntoIterator<Item = Tick<T>>>(&'l mut self, ticks: TL, tick_options: &[TickOption<'m>], label_options: &[LabelOption<'m>])
+		-> &'l mut Self
 	{
 		self.get_common_data_mut().y_axis.set_ticks_custom(ticks, tick_options, label_options);
 		self
 	}
 
 	/// Like `set_x_ticks_custom` but for the the color bar axis.
-	fn set_cb_ticks_custom<'l, T: DataType, TL: IntoIterator<Item = Tick<T>>>(&'l mut self, ticks: TL, tick_options: &[TickOption], label_options: &[LabelOption]) -> &'l mut Self
+	fn set_cb_ticks_custom<'l, T: DataType, TL: IntoIterator<Item = Tick<T>>>(&'l mut self, ticks: TL, tick_options: &[TickOption<'m>], label_options: &[LabelOption<'m>])
+		-> &'l mut Self
 	{
 		self.get_common_data_mut().cb_axis.set_ticks_custom(ticks, tick_options, label_options);
 		self
@@ -1282,21 +1332,10 @@ pub trait AxesCommon : AxesCommonPrivate
 	///      * `Color` - Specifies the color of the grid lines
 	///      * `LineStyle` - Specifies the style of the grid lines
 	///      * `LineWidth` - Specifies the width of the grid lines
-	fn set_grid_options<'l>(&'l mut self, front: bool, options: &[PlotOption]) -> &'l mut Self
+	fn set_grid_options<'l>(&'l mut self, front: bool, options: &[PlotOption<'m>]) -> &'l mut Self
 	{
-		{
-			let c = &mut self.get_common_data_mut().grid_options as &mut Writer;
-			if front
-			{
-				c.write_str("front ");
-			}
-			else
-			{
-				c.write_str("back ");
-			}
-			AxesCommonData::write_line_options(c, options);
-			AxesCommonData::write_color_options(c, options, None);
-		}
+		self.get_common_data_mut().grid_front = front;
+		self.get_common_data_mut().grid_options = options.into();
 		self
 	}
 
@@ -1314,20 +1353,27 @@ pub trait AxesCommon : AxesCommonPrivate
 				{
 					assert!(gamma > 0.0, "Gamma must be positive");
 					writeln!(c, "set palette gray gamma {:.12e}", gamma);
-				},
+				}
 				Formula(r, g, b) =>
 				{
 					assert!(r >= -36 && r <= 36, "Invalid r formula!");
 					assert!(g >= -36 && g <= 36, "Invalid g formula!");
 					assert!(b >= -36 && b <= 36, "Invalid b formula!");
 					writeln!(c, "set palette rgbformulae {},{},{}", r, g, b);
-				},
+				}
 				CubeHelix(start, rev, sat, gamma) =>
 				{
 					assert!(sat >= 0.0, "Saturation must be non-negative");
 					assert!(gamma > 0.0, "Gamma must be positive");
-					writeln!(c, "set palette cubehelix start {:.12e} cycles {:.12e} saturation {:.12e} gamma {:.12e}", start, rev, sat, gamma);
-				},
+					writeln!(
+						c,
+						"set palette cubehelix start {:.12e} cycles {:.12e} saturation {:.12e} gamma {:.12e}",
+						start,
+						rev,
+						sat,
+						gamma
+					);
+				}
 			}
 		}
 		self
