@@ -1,10 +1,10 @@
 // Copyright (c) 2013-2014 by SiegeLord
-// 
+//
 // All rights reserved. Distributed under LGPL 3.0. For full terms see the file LICENSE.
 
 macro_rules! first_opt
 {
-	($O: ident , $P: pat => $B: expr ) =>
+	($O: expr , $P: pat => $B: expr ) =>
 	(
 		for o in $O.iter()
 		{
@@ -23,7 +23,7 @@ macro_rules! first_opt
 
 macro_rules! first_opt_default
 {
-	($O: ident , $P: pat => $B: expr , _ => $E: expr ) =>
+	($O: expr , $P: pat => $B: expr , _ => $E: expr ) =>
 	(
 		{
 			let mut found = false;
